@@ -1,7 +1,7 @@
-//! This crate allows interacting with the data stored internally by [`OsStr`]
-//! and [`OsString`], without resorting to panics or corruption for invalid
-//! UTF-8. Thus, methods can be used that are already defined on
-//! [`[u8]`][slice] and [`Vec<u8>`].
+//! This crate allows interacting with the data stored by [`OsStr`] and
+//! [`OsString`], without resorting to panics or corruption for invalid UTF-8.
+//! Thus, methods can be used that are already defined on [`[u8]`][slice] and
+//! [`Vec<u8>`].
 //!
 //! Typically, the only way to losslessly construct [`OsStr`] or [`OsString`]
 //! from a byte sequence is to use `OsStr::new(str::from_utf8(bytes)?)`, which
@@ -36,9 +36,12 @@
 //! Additionally, concatenation may yield unexpected results without a UTF-8
 //! separator. If two platform strings need to be concatenated, the only safe
 //! way to do so is using [`OsString::push`]. This limitation also makes it
-//! undesirable to use the bytes in interchange unless absolutely necessary. If
-//! the strings need to be written as output, crate [print\_bytes] can do so
-//! more safely than directly writing the bytes.
+//! undesirable to use the bytes in interchange.
+//!
+//! Since this encoding can change between versions and platforms, it should
+//! not be used for storage. The standard library provides implementations of
+//! [`OsStrExt`] and [`OsStringExt`] for various platforms, which should be
+//! preferred for that use case.
 //!
 //! # User Input
 //!
@@ -61,15 +64,23 @@
 //! These features are optional and can be enabled or disabled in a
 //! "Cargo.toml" file.
 //!
+//! ### Default Features
+//!
+//! - **memchr** -
+//!   Changes the implementation to use crate [memchr] for better performance.
+//!   This feature is useless when "raw\_os\_str" is disabled.
+//!
+//!   For more information, see [`RawOsStr`][memchr complexity].
+//!
+//! - **raw\_os\_str** -
+//!   Enables use of [`RawOsStr`] and [`RawOsString`].
+//!
 //! ### Optional Features
 //!
-//! - **print_bytes** -
+//! - **print\_bytes** -
 //!   Provides implementations of [`print_bytes::ToBytes`] for [`RawOsStr`] and
 //!   [`RawOsString`].
 //!
-//! - **raw_os_str** -
-//!   Enables use of [`RawOsStr`] and [`RawOsString`].
-//!
 //! - **uniquote** -
 //!   Provides implementations of [`uniquote::Quote`] for [`RawOsStr`] and
 //!   [`RawOsString`].
@@ -128,6 +139,10 @@
 //! [bstr]: https://crates.io/crates/bstr
 //! [`ByteSlice::to_os_str`]: https://docs.rs/bstr/0.2.12/bstr/trait.ByteSlice.html#method.to_os_str
 //! [`ByteVec::into_os_string`]: https://docs.rs/bstr/0.2.12/bstr/trait.ByteVec.html#method.into_os_string
+//! [memchr complexity]: RawOsStr#complexity
+//! [memchr]: https://crates.io/crates/memchr
+//! [`OsStrExt`]: ::std::os::unix::ffi::OsStrExt
+//! [`OsStringExt`]: ::std::os::unix::ffi::OsStringExt
 //! [sealed]: https://rust-lang.github.io/api-guidelines/future-proofing.html#c-sealed
 //! [print\_bytes]: https://crates.io/crates/print_bytes
 
@@ -155,8 +170,6 @@ use std::path::Path;
 use std::path::PathBuf;
 use std::result;
 
-// https://github.com/rust-lang/rust/issues/88453
-#[doc(hidden)]
 macro_rules! if_raw_str {
     ( $($item:item)+ ) => {
         $(
@@ -167,21 +180,12 @@ macro_rules! if_raw_str {
 }
 
 #[cfg_attr(
-    all(
-        target_arch = "wasm32",
-        any(target_os = "emscripten", target_os = "unknown"),
-    ),
+    all(target_arch = "wasm32", target_os = "unknown"),
     path = "wasm32/mod.rs"
 )]
 #[cfg_attr(windows, path = "windows/mod.rs")]
 #[cfg_attr(
-    not(any(
-        all(
-            target_arch = "wasm32",
-            any(target_os = "emscripten", target_os = "unknown"),
-        ),
-        windows,
-    )),
+    not(any(all(target_arch = "wasm32", target_os = "unknown"), windows)),
     path = "common/mod.rs"
 )]
 mod imp;
