@@ -3,6 +3,10 @@ use std::borrow::Cow;
 use std::borrow::ToOwned;
 use std::ffi::OsStr;
 use std::ffi::OsString;
+use std::fmt;
+use std::fmt::Debug;
+use std::fmt::Display;
+use std::fmt::Formatter;
 use std::mem;
 use std::ops::Deref;
 use std::ops::Index;
@@ -14,6 +18,11 @@ use std::ops::RangeTo;
 use std::ops::RangeToInclusive;
 use std::str;
 
+#[cfg(feature = "memchr")]
+use memchr::memmem::find;
+#[cfg(feature = "memchr")]
+use memchr::memmem::rfind;
+
 use super::imp::raw;
 use super::iter::Split;
 use super::pattern::Encoded as EncodedPattern;
@@ -21,15 +30,8 @@ use super::OsStrBytes;
 use super::OsStringBytes;
 use super::Pattern;
 
-#[cfg(feature = "print_bytes")]
-use print_bytes::Bytes;
-#[cfg(feature = "print_bytes")]
-use print_bytes::ToBytes;
-
-#[cfg(feature = "uniquote")]
-use uniquote::Quote;
-
-fn find_pattern(string: &[u8], pat: &[u8]) -> Option<usize> {
+#[cfg(not(feature = "memchr"))]
+fn find(string: &[u8], pat: &[u8]) -> Option<usize> {
     for i in 0..=string.len().checked_sub(pat.len())? {
         if string[i..].starts_with(pat) {
             return Some(i);
@@ -38,7 +40,8 @@ fn find_pattern(string: &[u8], pat: &[u8]) -> Option<usize> {
     None
 }
 
-fn rfind_pattern(string: &[u8], pat: &[u8]) -> Option<usize> {
+#[cfg(not(feature = "memchr"))]
+fn rfind(string: &[u8], pat: &[u8]) -> Option<usize> {
     for i in (pat.len()..=string.len()).rev() {
         if string[..i].ends_with(pat) {
             return Some(i - pat.len());
@@ -56,9 +59,6 @@ macro_rules! impl_trim_matches {
         }
 
         let mut string = &$self.0;
-        #[allow(unused_mut)]
-        #[allow(unused_variables)]
-        let mut matches = 0;
         while let Some(substring) = string.$strip_method(pat) {
             string = substring;
         }
@@ -103,8 +103,15 @@ macro_rules! impl_split_once_raw {
 /// relying on this implementation detail is discouraged. Platform-specific
 /// indices are error-prone.
 ///
+/// # Complexity
+///
+/// All searching methods have worst-case multiplicative time complexity (i.e.,
+/// `O(self.raw_len() * pat.len())`). Enabling the "memchr" feature allows
+/// these methods to instead run in linear time in the worst case (documented
+/// for [`memchr::memmem::find`]).
+///
 /// [unspecified encoding]: super#encoding
-#[derive(Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Eq, Hash, Ord, PartialEq, PartialOrd)]
 #[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "raw_os_str")))]
 #[repr(transparent)]
 pub struct RawOsStr([u8]);
@@ -292,7 +299,7 @@ impl RawOsStr {
         let pat = pat.__encode();
         let pat = pat.__get();
 
-        find_pattern(&self.0, pat)
+        find(&self.0, pat)
     }
 
     /// Equivalent to [`str::is_empty`].
@@ -361,14 +368,14 @@ impl RawOsStr {
         let pat = pat.__encode();
         let pat = pat.__get();
 
-        rfind_pattern(&self.0, pat)
+        rfind(&self.0, pat)
     }
 
     pub(super) fn rsplit_once_raw<P>(&self, pat: &P) -> Option<(&Self, &Self)>
     where
         P: EncodedPattern,
     {
-        impl_split_once_raw!(self, pat, rfind_pattern)
+        impl_split_once_raw!(self, pat, rfind)
     }
 
     /// Equivalent to [`str::rsplit_once`].
@@ -489,7 +496,7 @@ impl RawOsStr {
     where
         P: EncodedPattern,
     {
-        impl_split_once_raw!(self, pat, find_pattern)
+        impl_split_once_raw!(self, pat, find)
     }
 
     /// Equivalent to [`str::split_once`].
@@ -808,24 +815,6 @@ r#impl!(RangeInclusive<usize>, x, *x.start(), x.end().wrapping_add(1));
 r#impl!(RangeTo<usize>, x, x.end);
 r#impl!(RangeToInclusive<usize>, x, x.end.wrapping_add(1));
 
-#[cfg(feature = "uniquote")]
-#[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "uniquote")))]
-impl Quote for RawOsStr {
-    #[inline]
-    fn escape(&self, f: &mut uniquote::Formatter<'_>) -> uniquote::Result {
-        self.0.escape(f)
-    }
-}
-
-#[cfg(feature = "print_bytes")]
-#[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "print_bytes")))]
-impl ToBytes for RawOsStr {
-    #[inline]
-    fn to_bytes(&self) -> Bytes<'_> {
-        self.0.to_bytes()
-    }
-}
-
 impl ToOwned for RawOsStr {
     type Owned = RawOsString;
 
@@ -840,7 +829,7 @@ impl ToOwned for RawOsStr {
 /// For more information, see [`RawOsStr`].
 ///
 /// [unspecified encoding]: super#encoding
-#[derive(Clone, Debug, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
 #[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "raw_os_str")))]
 pub struct RawOsString(Vec<u8>);
 
@@ -1012,23 +1001,62 @@ r#impl!(RangeInclusive<usize>);
 r#impl!(RangeTo<usize>);
 r#impl!(RangeToInclusive<usize>);
 
-#[cfg(feature = "uniquote")]
-#[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "uniquote")))]
-impl Quote for RawOsString {
-    #[inline]
-    fn escape(&self, f: &mut uniquote::Formatter<'_>) -> uniquote::Result {
-        (**self).escape(f)
+struct Buffer<'a>(&'a [u8]);
+
+impl Debug for Buffer<'_> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.write_str("\"")?;
+
+        let mut string = self.0;
+        let mut invalid_length = 0;
+        while !string.is_empty() {
+            let (invalid, substring) = string.split_at(invalid_length);
+
+            let valid = match str::from_utf8(substring) {
+                Ok(valid) => {
+                    string = &[];
+                    valid
+                }
+                Err(error) => {
+                    let (valid, substring) =
+                        substring.split_at(error.valid_up_to());
+
+                    let invalid_char_length =
+                        error.error_len().unwrap_or_else(|| substring.len());
+                    if valid.is_empty() {
+                        invalid_length += invalid_char_length;
+                        continue;
+                    }
+                    string = substring;
+                    invalid_length = invalid_char_length;
+
+                    // SAFETY: This slice was validated to be UTF-8.
+                    unsafe { str::from_utf8_unchecked(valid) }
+                }
+            };
+
+            raw::debug(invalid, f)?;
+            Display::fmt(&valid.escape_debug(), f)?;
+        }
+
+        f.write_str("\"")
     }
 }
 
-#[cfg(feature = "print_bytes")]
-#[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "print_bytes")))]
-impl ToBytes for RawOsString {
-    #[inline]
-    fn to_bytes(&self) -> Bytes<'_> {
-        (**self).to_bytes()
-    }
+macro_rules! r#impl {
+    ( $type:ty ) => {
+        impl Debug for $type {
+            #[inline]
+            fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+                f.debug_tuple(stringify!($type))
+                    .field(&Buffer(&self.0))
+                    .finish()
+            }
+        }
+    };
 }
+r#impl!(RawOsStr);
+r#impl!(RawOsString);
 
 macro_rules! r#impl {
     ( $type:ty , $other_type:ty ) => {
@@ -1057,3 +1085,54 @@ r#impl!(&RawOsStr, String);
 r#impl!(RawOsString, str);
 r#impl!(RawOsString, &str);
 r#impl!(RawOsString, String);
+
+#[cfg(feature = "print_bytes")]
+#[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "print_bytes")))]
+mod print_bytes {
+    use print_bytes::Bytes;
+    use print_bytes::ToBytes;
+
+    use super::RawOsStr;
+    use super::RawOsString;
+
+    impl ToBytes for RawOsStr {
+        #[inline]
+        fn to_bytes(&self) -> Bytes<'_> {
+            self.0.to_bytes()
+        }
+    }
+
+    impl ToBytes for RawOsString {
+        #[inline]
+        fn to_bytes(&self) -> Bytes<'_> {
+            (**self).to_bytes()
+        }
+    }
+}
+
+#[cfg(feature = "uniquote")]
+#[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "uniquote")))]
+mod uniquote {
+    use uniquote::Formatter;
+    use uniquote::Quote;
+    use uniquote::Result;
+
+    use crate::imp::raw;
+
+    use super::RawOsStr;
+    use super::RawOsString;
+
+    impl Quote for RawOsStr {
+        #[inline]
+        fn escape(&self, f: &mut Formatter<'_>) -> Result {
+            raw::uniquote::escape(&self.0, f)
+        }
+    }
+
+    impl Quote for RawOsString {
+        #[inline]
+        fn escape(&self, f: &mut Formatter<'_>) -> Result {
+            (**self).escape(f)
+        }
+    }
+}
